
/**
 * @vue-cdk/pluginify
 * Version: 0.4.7,
 * (c) Christian Kienle, 2020
 * LICENCE: MIT
 * https://github.com/ChristianKienle/cdk
*/
function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

// @ts-check

/** @type {import('./types/options').GetComponentName} */
// By default we prefix all of our components with a 'C'. A prefix is not needed in 99% of all cases but there are some situations which require a prefix. One edge case is the 'Link' component. 'Link' is considered by some players in the Vue ecosystem to be an invalid component name. For example having the following component:

/*
<template>
  <CLink>hi</CLink>
</template>
*/
// Will make eslint complain ('The template root requires an element rather than texts' and 'Parsing error: x-invalid-end-tag'). Because of that we simply prefix all of our components with a 'C'.
var defaultComponentName = function defaultComponentName(context) {
  return "C".concat(context.name);
};

// @ts-check
/** @returns {import('./types/options')._Options} */

var createDefaultOptions = (function () {
  return {
    onDidRegisterComponent: function onDidRegisterComponent() {},
    componentName: defaultComponentName
  };
});

var normalizedPluginOptions = (function () {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : createDefaultOptions();
  return _objectSpread2({}, createDefaultOptions(), {}, options);
});

// @ts-check

var getComponentName = function getComponentName(component) {
  if (typeof component === 'function') {
    var prototype = component.prototype;
    return prototype.constructor.extendOptions.name;
  }

  return component.name;
}; // This function turns any constructor into a Vue plugin.
// If you have a components that depends on other components to be globally
// installed pass them as the seoncond parameter.

/** @param {Array<import('vue').Component>} dependencies */


var index = (function () {
  for (var _len = arguments.length, dependencies = new Array(_len), _key = 0; _key < _len; _key++) {
    dependencies[_key] = arguments[_key];
  }

  var install = function install(vue, options) {
    var _options = normalizedPluginOptions(options);

    var onDidRegisterComponent = _options.onDidRegisterComponent;
    dependencies.forEach(function (component) {
      var rawComponentName = getComponentName(component);

      if (rawComponentName == null) {
        throw new Error("\n            Unable to determine component name. Component: ".concat(component, ". Did you forget to add a 'name' attribute?\n                "));
      }

      var componentName = _options.componentName({
        name: rawComponentName,
        component: component
      });

      vue.component(componentName, component);
      onDidRegisterComponent({
        component: component,
        name: componentName
      });
    });
  };

  function createMixin(options) {
    var _options = normalizedPluginOptions(options);

    var componentsOption = {};
    dependencies.forEach(function (component) {
      var rawComponentName = getComponentName(component);

      if (rawComponentName == null) {
        throw new Error("\n            Unable to determine component name. Component: ".concat(component, ". Did you forget to add a 'name' attribute?\n                "));
      }

      var componentName = _options.componentName({
        name: rawComponentName,
        component: component
      });

      componentsOption[componentName] = component;
    });
    return {
      components: componentsOption
    };
  }

  createMixin.install = install;
  return createMixin;
});

export default index;
